import Tkinter
import random
import time
import tkMessageBox
import winsound

class SimonMemoryGame:

    def __init__(self):

        self.colors = ['blue', 'red', 'yellow', 'green']        # list of colors
        self.winsounds = ['c1', 'e1', 'a1', 'c2']               # list of sounds in .wav format

        self.colorOrder = []        # fill in with the numeric equivalents of the colors randomly
        self.score = 0              # counter for the successfully completed rounds
        self.listOfPressedButtons = []      # fill in with the numeric equivalents of the colors after button press

        self.mainFrame = Tkinter.Tk()                   # create the top level figure called mainFrame
        self.mainFrame.title('Simon Memory Game')      # set the title of the figure to "Simon Memory Game"
        self.mainFrame.resizable(width=False, height=False) # fix the size of the figure

        self.buttons = ['blue', 'red', 'yellow', 'green']   # create the four colored buttons with a for loop
        for n in range(0, len(self.buttons), 1):
            self.buttons[n] = Tkinter.Button(self.mainFrame,  bg=self.colors[n], font=1, height=10, width=30, command=lambda color=n:(self.pushButton(color)))
            self.buttons[n].grid(row=n*2 % 4, column=n*2 / 4)

        self.lowerField = Tkinter.Canvas(self.mainFrame, bg='white', height=10)     # field for additional buttons or textfields
        self.lowerField.grid(row=3, columnspan=2)

        self.startButton = Tkinter.Button(self.lowerField, height=2, width=20, text='Start', command=lambda: self.start())  # button to start the game
        self.startButton.grid(row=0, column=0)

        self.scoreField = Tkinter.Label(self.lowerField, bg='white', height=2, width=20, text='Score: '+str(self.score))    # textfield to show the current score
        self.scoreField.grid(row=1, column=1)

        self.resetButton = Tkinter.Button(self.lowerField, height=2, width=20, text='Reset', command=lambda: self.reset())  # button to reset the game
        self.resetButton.grid(row=0, column=2)

        self.varCheckboxSounds = Tkinter.IntVar()
        self.checkboxSounds = Tkinter.Checkbutton(self.mainFrame, text="Sounds", variable=self.varCheckboxSounds, height=3, width=20)   # checkbox for activating sounds
        self.checkboxSounds.grid(row=1, column=0)

        self.varCheckboxStroop = Tkinter.IntVar()
        self.checkboxStroop = Tkinter.Checkbutton(self.mainFrame, text="Stroop Effect", variable=self.varCheckboxStroop, height=3, width=20)    # checkbox for activating the stroop effect
        self.checkboxStroop.grid(row=1, column=1)

# defining the method pushbutton, that is the callback for pushing one of the colored buttons
    def pushButton(self, color):                                        # the colors are here the numbers from 0 to 3
        if self.varCheckboxSounds.get() == 1:                           # check, if the sounds are activated
            winsound.PlaySound(self.winsounds[color], winsound.SND_NODEFAULT)   # play sound

        self.listOfPressedButtons.append(color)                         # extend the listOfPressedButtons
        for counter in range(0, len(self.listOfPressedButtons), 1):     # going through the listOfPressedButtons
            if self.listOfPressedButtons[counter] == self.colorOrder[counter]:   # if the button color is equal to the current color at every entry
                print 'right button'

            else:
                print 'wrong input'
                self.reset()            # call the reset method
                break

        if len(self.colorOrder) == len(self.listOfPressedButtons):  # if the length of the lists are equal = all colors are correct
            self.updateScore()                  # call method to update the score
            self.mainFrame.update()
            self.newColor()                     # call method to randomly add a new color
            self.listOfPressedButtons = []      # empty the listOfPressedButtons
            time.sleep(0.2)                     # wait for some time
            self.showColorOrder()               # call the method to show the new color order in the figure

# defining the method to show the color order in the figure
    def showColorOrder(self):
        for numericEntries in self.colorOrder:                          # for all numeric color entries (between 0 and 3) of the list "colorOrder"
            self.buttons[numericEntries].flash()                        # flash of the proper button
            if self.varCheckboxSounds.get() == 1:                       # check, if the sounds are activated
                winsound.PlaySound(self.winsounds[numericEntries], winsound.SND_NODEFAULT)  # play sound
            if self.varCheckboxSounds.get() == 0:
                time.sleep(0.2)                                             # wait for some time


# defining the method to reset the game
    def reset(self):
        tkMessageBox.showwarning('Game Over', 'Game over! \n'                               # show a message box
                                              'Your Score: ' + str(self.score) + '\n'
                                              'Please click on start to restart the game')
        self.colorOrder = []            # empty the list of the colors
        self.score = 0                  # set the score to zero
        self.scoreField.config(text='Score: ' + str(self.score))    # update the new score in the text field

# defining the method to start the game
    def start(self):
        self.newColor()                     # call the method to randomly add a new color
        self.showColorOrder()               # call the method to show the new color order in the figure
        self.listOfPressedButtons = []      # empty the listOfPressedButtons
        print 'start'

# defining the method to randomly add a new color
    def newColor(self):
        self.nextColor = random.randint(0, 3)       # randomly generate a number between 0 and 3 (--> reference to colors)
        self.colorOrder.append(self.nextColor)      # add the new color to the list "colorOrder"
        self.stroopEffect()                         # call the method for the stroop effect

# defining the method to update the score in the textfield
    def updateScore(self):
        self.score += 1                                             # increment the score by one
        self.scoreField.config(text='Score: '+str(self.score))      # update the textfield

# defining the method for activating the stroop effect
    def stroopEffect(self):
        if self.varCheckboxStroop.get() == 1:               # if the checkbox is set
            for n in range(0, len(self.buttons), 1):
                self.buttons[n].config(bg='black', fg=self.colors[n], text=random.choice(self.colors))     # set the text in the colored buttons to a random color
        if self.varCheckboxStroop.get() == 0:               # if the checkbox is cleared
            for n in range(0, len(self.buttons), 1):
                self.buttons[n].config(bg=self.colors[n], text='')                             # set the text in the colored buttons to an empty string
        self.mainFrame.update()
game = SimonMemoryGame()
game.mainFrame.mainloop()


# to do:
# - sounds (checkbox einfuegen) --> Tkinter Checkbutton - other sounds at C:Windows/Media
# - Stroop Effect --> font vergroessern

# n=0 --> blue
# n=1 --> red
# n=2 --> yellow
# n=3 --> green